@@ -133,6 +133,7 @@ namespace Extension
 		virtual bool operator==(const RegexParametresElement& element) const = 0;
 		virtual RegexParametresElementSP copie() const = 0;
 		virtual bool autre() const { return false; }
+		virtual char type() const = 0;
 		bool _modifiable;
 	};
 
@@ -156,16 +157,16 @@ namespace Extension
 		}
 		virtual bool operator<(const RegexParametresElement& element) const override
 		{
-			if(typeid(*this).hash_code()!=typeid(element).hash_code())
-			{
-				return typeid(*this).hash_code()<typeid(element).hash_code();
-			}
+			auto t = type();
+			auto tt = element.type();
+			if(t!=tt) { return t<tt; }
 			return false;
 		}
 		virtual bool operator==(const RegexParametresElement& element) const override
 		{
-			return (typeid(*this).hash_code()==typeid(element).hash_code());
+			return (type()==element.type());
 		}
+		virtual char type() const { return 0; }
 	};
 /*	Abandonne, car impossible avec les jokers
 	struct RegexParametresAutre : public RegexParametresElement
@@ -196,17 +197,17 @@ namespace Extension
 		}
 		virtual bool operator<(const RegexParametresElement& element) const override
 		{
-			if(typeid(*this).hash_code()!=typeid(element).hash_code())
-			{
-				return typeid(*this).hash_code()<typeid(element).hash_code();
-			}
+			auto t = type();
+			auto tt = element.type();
+			if(t!=tt) { return t<tt; }
 			return false;
 		}
 		virtual bool operator==(const RegexParametresElement& element) const override
 		{
-			return (typeid(*this).hash_code()==typeid(element).hash_code());
+			return (type()==element.type());
 		}
 		virtual bool autre() const override { return true; }
+		virtual char type() const { return -1; }
 		EnsembleExclus _exclus;
 	};
 */
@@ -234,10 +235,9 @@ namespace Extension
 		}
 		virtual bool operator<(const RegexParametresElement& element) const override
 		{
-			if(typeid(*this).hash_code()!=typeid(element).hash_code())
-			{
-				return typeid(*this).hash_code()<typeid(element).hash_code();
-			}
+			auto t = type();
+			auto tt = element.type();
+			if(t!=tt) { return t<tt; }
 			if(_modifiable xor element._modifiable)
 			{
 				return element._modifiable;
@@ -246,12 +246,13 @@ namespace Extension
 		}
 		virtual bool operator==(const RegexParametresElement& element) const override
 		{
-			if(typeid(*this).hash_code()!=typeid(element).hash_code())
+			if(type()!=element.type())
 			{
 				return false;
 			}
 			return (_modifiable == element._modifiable);
 		}
+		virtual char type() const { return 1; }
 	};
 
 	DECL_SHARED_PTR(RegexParametresValeur);
@@ -283,20 +284,19 @@ namespace Extension
 		}
 		virtual bool operator<(const RegexParametresElement& element) const override
 		{
-			if(typeid(*this).hash_code()!=typeid(element).hash_code())
-			{
-				return typeid(*this).hash_code()<typeid(element).hash_code();
-			}
+			auto t = type();
+			auto tt = element.type();
+			if(t!=tt) { return t<tt; }
 			if(_modifiable xor element._modifiable)
 			{
 				return element._modifiable;
 			}
-			auto e = dynamic_cast<const RegexParametresValeur&>(element);
+			auto e = static_cast<const RegexParametresValeur&>(element);
 			return _type<e._type;
 		}
 		virtual bool operator==(const RegexParametresElement& element) const override
 		{
-			if(typeid(*this).hash_code()!=typeid(element).hash_code())
+			if(type()!=element.type())
 			{
 				return false;
 			}
@@ -304,9 +304,10 @@ namespace Extension
 			{
 				return false;
 			}
-			auto e = dynamic_cast<const RegexParametresValeur&>(element);
+			auto e = static_cast<const RegexParametresValeur&>(element);
 			return _type==e._type;
 		}
+		virtual char type() const { return 2; }
 		SVM_Memoire::Type _type;
 	};
 
@@ -330,16 +331,16 @@ namespace Extension
 		}
 		virtual bool operator<(const RegexParametresElement& element) const override
 		{
-			if(typeid(*this).hash_code()!=typeid(element).hash_code())
-			{
-				return typeid(*this).hash_code()<typeid(element).hash_code();
-			}
+			auto t = type();
+			auto tt = element.type();
+			if(t!=tt) { return t<tt; }
 			return false;
 		}
 		virtual bool operator==(const RegexParametresElement& element) const override
 		{
-			return (typeid(*this).hash_code()==typeid(element).hash_code());
+			return (type()==element.type());
 		}
+		virtual char type() const { return 3; }
 	};
 
 	struct RegexParametresMarqueur : public RegexParametresElement
@@ -366,22 +367,22 @@ namespace Extension
 		}
 		virtual bool operator<(const RegexParametresElement& element) const override
 		{
-			if(typeid(*this).hash_code()!=typeid(element).hash_code())
-			{
-				return typeid(*this).hash_code()<typeid(element).hash_code();
-			}
-			auto e = dynamic_cast<const RegexParametresMarqueur&>(element);
+			auto t = type();
+			auto tt = element.type();
+			if(t!=tt) { return t<tt; }
+			auto e = static_cast<const RegexParametresMarqueur&>(element);
 			return _marqueur<e._marqueur;
 		}
 		virtual bool operator==(const RegexParametresElement& element) const override
 		{
-			if(typeid(*this).hash_code()!=typeid(element).hash_code())
+			if(type()!=element.type())
 			{
 				return false;
 			}
-			auto e = dynamic_cast<const RegexParametresMarqueur&>(element);
+			auto e = static_cast<const RegexParametresMarqueur&>(element);
 			return _marqueur==e._marqueur;
 		}
+		virtual char type() const { return 4; }
 		SVM_Programme::ParametreMarqueur _marqueur;
 	};
 
@@ -405,16 +406,16 @@ namespace Extension
 		}
 		virtual bool operator<(const RegexParametresElement& element) const override
 		{
-			if(typeid(*this).hash_code()!=typeid(element).hash_code())
-			{
-				return typeid(*this).hash_code()<typeid(element).hash_code();
-			}
+			auto t = type();
+			auto tt = element.type();
+			if(t!=tt) { return t<tt; }
 			return false;
 		}
 		virtual bool operator==(const RegexParametresElement& element) const override
 		{
-			return (typeid(*this).hash_code()==typeid(element).hash_code());
+			return (type()==element.type());
 		}
+		virtual char type() const { return 5; }
 	};
 
 	struct RegexParametresMotClef : public RegexParametresElement
@@ -439,22 +440,22 @@ namespace Extension
 		}
 		virtual bool operator<(const RegexParametresElement& element) const override
 		{
-			if(typeid(*this).hash_code()!=typeid(element).hash_code())
-			{
-				return typeid(*this).hash_code()<typeid(element).hash_code();
-			}
-			auto e = dynamic_cast<const RegexParametresMotClef&>(element);
+			auto t = type();
+			auto tt = element.type();
+			if(t!=tt) { return t<tt; }
+			auto e = static_cast<const RegexParametresMotClef&>(element);
 			return _mot_clef<e._mot_clef;
 		}
 		virtual bool operator==(const RegexParametresElement& element) const override
 		{
-			if(typeid(*this).hash_code()!=typeid(element).hash_code())
+			if(type()!=element.type())
 			{
 				return false;
 			}
-			auto e = dynamic_cast<const RegexParametresMotClef&>(element);
+			auto e = static_cast<const RegexParametresMotClef&>(element);
 			return _mot_clef==e._mot_clef;
 		}
+		virtual char type() const { return 6; }
 		std::string _mot_clef;
 	};
 
@@ -482,10 +483,9 @@ namespace Extension
 		}
 		virtual bool operator<(const RegexParametresElement& element) const override
 		{
-			if(typeid(*this).hash_code()!=typeid(element).hash_code())
-			{
-				return typeid(*this).hash_code()<typeid(element).hash_code();
-			}
+			auto t = type();
+			auto tt = element.type();
+			if(t!=tt) { return t<tt; }
 			if(_modifiable xor element._modifiable)
 			{
 				return element._modifiable;
@@ -494,12 +494,13 @@ namespace Extension
 		}
 		virtual bool operator==(const RegexParametresElement& element) const override
 		{
-			if(typeid(*this).hash_code()!=typeid(element).hash_code())
+			if(type()!=element.type())
 			{
 				return false;
 			}
 			return (_modifiable == element._modifiable);
 		}
+		virtual char type() const { return 7; }
 	};
 
 	struct RegexParametresJokerPartielVariable : public RegexParametresElement
@@ -522,16 +523,16 @@ namespace Extension
 		}
 		virtual bool operator<(const RegexParametresElement& element) const override
 		{
-			if(typeid(*this).hash_code()!=typeid(element).hash_code())
-			{
-				return typeid(*this).hash_code()<typeid(element).hash_code();
-			}
+			auto t = type();
+			auto tt = element.type();
+			if(t!=tt) { return t<tt; }
 			return false;
 		}
 		virtual bool operator==(const RegexParametresElement& element) const override
 		{
-			return (typeid(*this).hash_code()==typeid(element).hash_code());
+			return (type()==element.type());
 		}
+		virtual char type() const { return 8; }
 	};
 
 	DECL_SHARED_PTR(RegexParametresStructure);
@@ -563,20 +564,19 @@ namespace Extension
 		}
 		virtual bool operator<(const RegexParametresElement& element) const override
 		{
-			if(typeid(*this).hash_code()!=typeid(element).hash_code())
-			{
-				return typeid(*this).hash_code()<typeid(element).hash_code();
-			}
+			auto t = type();
+			auto tt = element.type();
+			if(t!=tt) { return t<tt; }
 			if(_modifiable xor element._modifiable)
 			{
 				return element._modifiable;
 			}
-			auto e = dynamic_cast<const RegexParametresStructure&>(element);
+			auto e = static_cast<const RegexParametresStructure&>(element);
 			return _type<e._type;
 		}
 		virtual bool operator==(const RegexParametresElement& element) const override
 		{
-			if(typeid(*this).hash_code()!=typeid(element).hash_code())
+			if(type()!=element.type())
 			{
 				return false;
 			}
@@ -584,15 +584,16 @@ namespace Extension
 			{
 				return false;
 			}
-			auto e = dynamic_cast<const RegexParametresStructure&>(element);
+			auto e = static_cast<const RegexParametresStructure&>(element);
 			return _type==e._type;
 		}
+		virtual char type() const { return 9; }
 		SVM_Memoire::Type _type;
 	};
 
 	struct RegexParametresPasDeParametres : public RegexParametresElement
 	{
-		// Cette structure sert juste a indiquer qu'aucun parameter est
+		// Cette structure sert juste a indiquer qu'aucun parametre est
 		// possible dans les suggestions d'erreur sur les conformites
 		// d'automate. Seule la fonction de formattage est utilisable pour
 		// cette classe.
@@ -602,26 +603,12 @@ namespace Extension
 		{
 			os << "no parameter";
 		}
-		virtual RegexParametresElementSP joker() const override
-		{
-			throw;
-		}
-		virtual operator AutomateNonDeterministeSP () const override
-		{
-			throw;
-		}
-		virtual RegexParametresElementSP copie() const override
-		{
-			throw;
-		}
-		virtual bool operator<(const RegexParametresElement& element) const override
-		{
-			throw;
-		}
-		virtual bool operator==(const RegexParametresElement& element) const override
-		{
-			throw;
-		}
+		virtual RegexParametresElementSP joker() const override { throw; }
+		virtual operator AutomateNonDeterministeSP () const override { throw; }
+		virtual RegexParametresElementSP copie() const override { throw; }
+		virtual bool operator<(const RegexParametresElement& element) const override { throw; }
+		virtual bool operator==(const RegexParametresElement& element) const override { throw; }
+		virtual char type() const { return -2; }
 	};
 	
 	DECL_SHARED_PTR(RegexParametresAlternativeSimple);
