@@ -2,17 +2,17 @@
  * Simple Virtual Machine - A versatile and robust architecture to
  * easily write applications.
  * Copyright (C) 2021  Julien BRUGUIER
- * 
+ *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
@@ -169,7 +169,7 @@ struct Test
 	typedef std::map<std::string,Test> Liste;
 	Test(const std::string& commande, const Fichier& contenu)
 	:_commande(commande), _contenu(contenu) {}
-	
+
 	const std::string _commande;
 	const Fichier _contenu;
 };
@@ -224,6 +224,7 @@ struct Erreur
 		f << ": " << e._message << std::endl;
 		return f;
 	}
+	const std::string& message() const { return _message; }
 	private:
 		std::string _message;
 		size_t _debut;
@@ -406,7 +407,7 @@ struct DefinitionObjet : public Definition
 			<< _corps << std::endl
 			<< "#line" << std::endl;
 	}
-	
+
 	bool nouvelle_fonction(const std::string& nom, const FonctionSP& fonction)
 	{
 		if(not fonction)
@@ -473,6 +474,9 @@ struct DefinitionCode : public Definition
 
 // Pour svm_package
 
+struct ConfigurationPaquet;
+DECL_SHARED_PTR(ValeurOptionConstante);
+
 DECL_SHARED_PTR(ValeurOption);
 struct ValeurOption
 {
@@ -513,7 +517,16 @@ struct ValeurOption
 		return "";
 	}
 	virtual ~ValeurOption() {};
-	friend std::ostream& operator<<(std::ostream& os, const ValeurOption& v)
+	virtual ValeurOptionConstanteSP evaluation(ConfigurationPaquet& configuration) const = 0;
+	std::string evaluation_chaine(ConfigurationPaquet& configuration) const;
+	bool evaluation_booleene(ConfigurationPaquet& configuration) const;
+};
+
+struct ValeurOptionConstante : public ValeurOption
+{
+	virtual bool evaluation_booleene() const = 0;
+	virtual std::string evaluation_chaine() const = 0;
+	friend std::ostream& operator<<(std::ostream& os, const ValeurOptionConstante& v)
 	{
 		v.format(os);
 		return os;
@@ -522,60 +535,65 @@ struct ValeurOption
 };
 
 DECL_SHARED_PTR(ValeurOptionEntier);
-struct ValeurOptionEntier : public ValeurOption
+struct ValeurOptionEntier : public ValeurOptionConstante
 {
 	ValeurOptionEntier(const long int valeur)
 	:_valeur(valeur) {}
 	virtual ValeurOption::Type type() const override { return ValeurOption::Type::ENTIER; }
-	virtual void format(std::ostream& os) const
+	virtual ValeurOptionConstanteSP evaluation(ConfigurationPaquet& configuration) const override { return std::make_shared<ValeurOptionEntier>(_valeur); }
+	virtual bool evaluation_booleene() const override { return _valeur!=0; }
+	virtual std::string evaluation_chaine() const override
 	{
-		os << _valeur;
+		std::ostringstream oss;
+		oss << _valeur;
+		return oss.str();
 	}
+	virtual void format(std::ostream& os) const override { os << _valeur; }
+
 	long int _valeur;
 };
 
 DECL_SHARED_PTR(ValeurOptionChaine);
-struct ValeurOptionChaine : public ValeurOption
+struct ValeurOptionChaine : public ValeurOptionConstante
 {
 	explicit ValeurOptionChaine(const std::string& valeur)
 	:_valeur(valeur) {}
 	virtual ValeurOption::Type type() const override { return ValeurOption::Type::CHAINE; }
-	virtual void format(std::ostream& os) const
-	{
-		os << "\"" << _valeur << "\"";
-	}
+	virtual ValeurOptionConstanteSP evaluation(ConfigurationPaquet& configuration) const override { return std::make_shared<ValeurOptionChaine>(_valeur); }
+	virtual bool evaluation_booleene() const override { return not _valeur.empty(); }
+	virtual std::string evaluation_chaine() const override { return _valeur; }
+	virtual void format(std::ostream& os) const override { os << "\"" << _valeur << "\"";}
+
 	std::string _valeur;
 };
 
 DECL_SHARED_PTR(ValeurOptionBooleen);
-struct ValeurOptionBooleen : public ValeurOption
+struct ValeurOptionBooleen : public ValeurOptionConstante
 {
 	ValeurOptionBooleen(const bool valeur)
 	:_valeur(valeur) {}
 	virtual ValeurOption::Type type() const override { return ValeurOption::Type::BOOLEEN; }
-	virtual void format(std::ostream& os) const
+	virtual ValeurOptionConstanteSP evaluation(ConfigurationPaquet& configuration) const override { return std::make_shared<ValeurOptionBooleen>(_valeur); }
+	virtual bool evaluation_booleene() const override { return _valeur; }
+	virtual std::string evaluation_chaine() const override
 	{
-		if(_valeur)
-		{
-			os << "YES";
-		}
-		else
-		{
-			os << "NO";
-		}
+		if(_valeur) return "yes";
+		return "no";
 	}
+	virtual void format(std::ostream& os) const override { if(_valeur) { os << "YES"; } else { os << "NO"; } }
+
 	bool _valeur;
 };
 
 DECL_SHARED_PTR(ValeurOptionDefaut);
-struct ValeurOptionDefaut : public ValeurOption
+struct ValeurOptionDefaut : public ValeurOptionConstante
 {
 	ValeurOptionDefaut() = default;
 	virtual ValeurOption::Type type() const override { return ValeurOption::Type::DEFAUT; }
-	virtual void format(std::ostream& os) const
-	{
-		os << "DEFAULT";
-	}
+	virtual ValeurOptionConstanteSP evaluation(ConfigurationPaquet& configuration) const override { return std::make_shared<ValeurOptionDefaut>(); }
+	virtual bool evaluation_booleene() const override { return false; }
+	virtual std::string evaluation_chaine() const override { return ""; }
+	virtual void format(std::ostream& os) const override { os << "DEFAULT"; }
 };
 
 DECL_SHARED_PTR(ValeurOptionVariable);
@@ -585,16 +603,27 @@ struct ValeurOptionVariable : public ValeurOption
 	:_nom(nom) {}
 	virtual ~ValeurOptionVariable() {}
 	virtual ValeurOption::Type type() const override { return ValeurOption::Type::VARIABLE; }
-	virtual void format(std::ostream& os) const
-	{
-		os << _nom;
-	}
+	virtual ValeurOptionConstanteSP evaluation(ConfigurationPaquet& configuration) const override;
+
 	std::string _nom;
 };
 
+DECL_SHARED_PTR(ValeurOptionConcatenation);
+struct ValeurOptionConcatenation : public ValeurOption
+{
+	explicit ValeurOptionConcatenation(const ValeurOptionSP& gauche, const ValeurOptionSP droite)
+	:_gauche(gauche), _droite(droite) {}
+	virtual ~ValeurOptionConcatenation() {}
+	virtual ValeurOption::Type type() const override { return ValeurOption::Type::CHAINE; }
+	virtual ValeurOptionConstanteSP evaluation(ConfigurationPaquet& configuration) const override;
+
+	ValeurOptionSP _gauche;
+	ValeurOptionSP _droite;
+};
+
 struct DefinitionAttributPaquet
 {
-	typedef std::vector<ValeurOptionSP> ValeursParDefaut;
+	typedef std::vector<ValeurOptionConstanteSP> ValeursParDefaut;
 	typedef std::vector<std::set<ValeurOption::Type> > TypesAutorises;
 	std::string _type;
 	ValeursParDefaut _defaut;
@@ -607,7 +636,7 @@ struct AttributPaquet;
 struct ConfigurationPaquet
 {
 	ConfigurationPaquet();
-	void modifie(const AttributPaquet& a, const size_t position);
+	void modifie(const AttributPaquet& a);
 	ValeurOptionSP lit(const std::string& nom, const size_t indice_valeur) const;
 	long int lit_entier(const std::string& nom, const size_t indice_valeur) const;
 	std::string lit_chaine(const std::string& nom, const size_t indice_valeur) const;
@@ -633,6 +662,7 @@ struct ConfigurationPaquet
 		};
 		return instance;
 	}
+
 	static void details(std::ostream& os)
 	{
 		auto const& configuration = ConfigurationPaquet::configuration();
@@ -642,34 +672,24 @@ struct ConfigurationPaquet
 			for(size_t i=0 ; i<c.second._types.size() ; ++i)
 			{
 				os << "\tParameter " << (i+1) << ":" << std::endl
-				   << "\t\tAllowed values:";
+					<< "\t\tAllowed values:";
 				for(const auto& t:c.second._types[i])
 				{
 					os << " " << ValeurOption::type(t);
 				}
 				os << std::endl
-				   << "\t\tDefault value: " << (*c.second._defaut[i]) << std::endl;
+					<< "\t\tDefault value: " << (*c.second._defaut[i]) << std::endl;
 			}
 			os << std::endl;
 		}
 	}
-	friend std::ostream& operator<<(std::ostream& os, const ConfigurationPaquet& cp)
-	{
-		os << "Configuration paquet" << std::endl;
-		for(const auto& e:cp._etat)
-		{
-			os << "\t" << e.first << ":";
-			for(const auto& v: e.second)
-			{
-				os << " " << (*v);
-			}
-			os << std::endl;
-		}
-		os << "End" << std::endl;
-		return os;
-	}
-	std::map<std::string,std::vector<ValeurOptionSP> > _etat;
-	std::map<std::string,ValeurOptionSP> _variables;
+
+	void variables_automatiques(const bool debug, const std::string& saveur);
+	void drapeaux(const std::set<std::string>& drapeaux, const std::set<std::string>& drapeaux_leves);
+	bool _condition = true;
+	std::map<std::string,std::vector<ValeurOptionConstanteSP> > _etat;
+	std::map<std::string,ValeurOptionConstanteSP> _variables;
+	std::set<std::string> _fichiers;
 };
 
 struct AttributPaquet
@@ -692,18 +712,89 @@ struct OptionPaquet: private AttributPaquet
 	void verification();
 	void construction(ConfigurationPaquet& configuration);
 
-	friend std::ostream& operator<<(std::ostream& os, const OptionPaquet& o)
-	{
-		os << "Option " << o._nom;
-		for(const auto& v: o._valeurs)
-		{
-			os << " " << (*v);
-		}
-		return os;
-	}
 	size_t _position;
 };
 
+DECL_SHARED_PTR(PaquetCondition);
+struct PaquetCondition
+{
+	explicit PaquetCondition(const size_t position)
+	:_position(position) {}
+	virtual ~PaquetCondition() {}
+	virtual bool evaluation(ConfigurationPaquet& configuration) const = 0;
+	size_t _position;
+};
+
+DECL_SHARED_PTR(PaquetConditionVariable);
+struct PaquetConditionVariable : public PaquetCondition
+{
+	PaquetConditionVariable(const std::string& nom, const size_t position)
+	:PaquetCondition(position), _nom(nom) {}
+	virtual ~PaquetConditionVariable() {}
+	virtual bool evaluation(ConfigurationPaquet& configuration) const override;
+
+	std::string _nom;
+};
+
+DECL_SHARED_PTR(PaquetConditionNon);
+struct PaquetConditionNon : public PaquetCondition
+{
+	PaquetConditionNon(const PaquetConditionSP& condition, const size_t position)
+	:PaquetCondition(position), _condition(condition) {}
+	virtual ~PaquetConditionNon() {}
+	virtual bool evaluation(ConfigurationPaquet& configuration) const override
+	{
+		return not _condition->evaluation(configuration);
+	}
+	PaquetConditionSP _condition;
+};
+
+DECL_SHARED_PTR(PaquetConditionEt);
+struct PaquetConditionEt : public PaquetCondition
+{
+	PaquetConditionEt(const PaquetConditionSP& gauche, const PaquetConditionSP& droite, const size_t position)
+	:PaquetCondition(position), _gauche(gauche), _droite(droite) {}
+	virtual ~PaquetConditionEt() {}
+	virtual bool evaluation(ConfigurationPaquet& configuration) const override
+	{
+		bool g = _gauche->evaluation(configuration);
+		if(not g) return false;
+		return g and _droite->evaluation(configuration);
+	}
+	PaquetConditionSP _gauche;
+	PaquetConditionSP _droite;
+};
+
+DECL_SHARED_PTR(PaquetConditionOu);
+struct PaquetConditionOu : public PaquetCondition
+{
+	PaquetConditionOu(const PaquetConditionSP& gauche, const PaquetConditionSP& droite, const size_t position)
+	:PaquetCondition(position), _gauche(gauche), _droite(droite) {}
+	virtual ~PaquetConditionOu() {}
+	virtual bool evaluation(ConfigurationPaquet& configuration) const override
+	{
+		bool g = _gauche->evaluation(configuration);
+		if(g) return true;
+		return g or _droite->evaluation(configuration);
+	}
+	PaquetConditionSP _gauche;
+	PaquetConditionSP _droite;
+};
+
+DECL_SHARED_PTR(PaquetConditionOuExclusif);
+struct PaquetConditionOuExclusif : public PaquetCondition
+{
+	PaquetConditionOuExclusif(const PaquetConditionSP& gauche, const PaquetConditionSP& droite, const size_t position)
+	:PaquetCondition(position), _gauche(gauche), _droite(droite) {}
+	virtual ~PaquetConditionOuExclusif() {}
+	virtual bool evaluation(ConfigurationPaquet& configuration) const override
+	{
+		return _gauche->evaluation(configuration) xor _droite->evaluation(configuration);
+	}
+	PaquetConditionSP _gauche;
+	PaquetConditionSP _droite;
+};
+
 struct FichierPaquet
 {
 	explicit FichierPaquet(const std::string& nom)
@@ -731,13 +822,9 @@ struct ContenuPaquet
 	virtual void verification() = 0;
 	virtual void construction(ConfigurationPaquet& configuration) = 0;
 	virtual void generation(std::ostream& os) = 0;
-	friend std::ostream& operator<<(std::ostream& os, const ContenuPaquet& c)
-	{
-		c.format(os);
-		return os;
-	}
-	virtual void format(std::ostream& os) const = 0;
+
 	size_t _position;
+	bool _condition = true;
 };
 
 DECL_SHARED_PTR(ContenuPaquetApplication);
@@ -749,10 +836,6 @@ struct ContenuPaquetApplication : public ContenuPaquet, private FichierPaquet
 	virtual void verification() override;
 	virtual void construction(ConfigurationPaquet& configuration) override;
 	virtual void generation(std::ostream& os) override;
-	virtual void format(std::ostream& os) const
-	{
-		os << "Application \"" << _nom << "\"";
-	}
 
 	bool _changer_interpreteur;
 	std::string _racine_extensions_locales;
@@ -768,10 +851,6 @@ struct ContenuPaquetBibliotheque : public ContenuPaquet, private FichierPaquet
 	virtual void verification() override;
 	virtual void construction(ConfigurationPaquet& configuration) override;
 	virtual void generation(std::ostream& os) override;
-	virtual void format(std::ostream& os) const
-	{
-		os << "Bibliotheque \"" << _nom << "\"";
-	}
 };
 
 DECL_SHARED_PTR(ContenuPaquetExtension);
@@ -783,10 +862,6 @@ struct ContenuPaquetExtension : public ContenuPaquet, private FichierPaquet
 	virtual void verification() override;
 	virtual void construction(ConfigurationPaquet& configuration) override;
 	virtual void generation(std::ostream& os) override;
-	virtual void format(std::ostream& os) const
-	{
-		os << "Extension \"" << _nom << "\"";
-	}
 
 	bool _installation;
 	bool _garde_construction;
@@ -802,10 +877,6 @@ struct ContenuPaquetFichier : public ContenuPaquet, private FichierPaquet
 	virtual void verification() override;
 	virtual void construction(ConfigurationPaquet& configuration) override;
 	virtual void generation(std::ostream& os) override;
-	virtual void format(std::ostream& os) const
-	{
-		os << "Fichier \"" << _nom << "\"";
-	}
 
 	std::string _mode;
 };
@@ -818,14 +889,6 @@ struct ContenuPaquetConfiguration : public ContenuPaquet, private AttributPaquet
 	virtual void verification() override;
 	virtual void construction(ConfigurationPaquet& configuration) override;
 	virtual void generation(std::ostream& os) override {}
-	virtual void format(std::ostream& os) const
-	{
-		os << "Configuration " << _nom;
-		for(const auto& v: _valeurs)
-		{
-			os << " " << (*v);
-		}
-	}
 };
 
 DECL_SHARED_PTR(ContenuPaquetVariable);
@@ -836,14 +899,6 @@ struct ContenuPaquetVariable : public ContenuPaquet, private AttributPaquet
 	virtual void verification() override {};
 	virtual void construction(ConfigurationPaquet& configuration) override;
 	virtual void generation(std::ostream& os) override {}
-	virtual void format(std::ostream& os) const
-	{
-		os << "Variable " << _nom;
-		for(const auto& v: _valeurs)
-		{
-			os << " " << (*v);
-		}
-	}
 };
 
 DECL_SHARED_PTR(ContenuPaquetCode);
@@ -855,11 +910,32 @@ struct ContenuPaquetCode : public ContenuPaquet
 	virtual void verification() override {}
 	virtual void construction(ConfigurationPaquet& configuration) override;
 	virtual void generation(std::ostream& os) override;
-	virtual void format(std::ostream& os) const
-	{
-		os << "Code \"" << _code << "\"";
-	}
 	std::string _code;
+
+	std::map<std::string,ValeurOptionConstanteSP> _variables;
+};
+
+DECL_SHARED_PTR(ContenuPaquetCondition);
+struct ContenuPaquetCondition : public ContenuPaquet
+{
+	ContenuPaquetCondition(const PaquetConditionSP& condition, const size_t position)
+	:ContenuPaquet(position), _condition(condition) {}
+	virtual ~ContenuPaquetCondition() {}
+	virtual void verification() override {}
+	virtual void construction(ConfigurationPaquet& configuration) override { configuration._condition = _condition->evaluation(configuration); }
+	virtual void generation(std::ostream& os) override {};
+	PaquetConditionSP _condition;
+};
+
+DECL_SHARED_PTR(ContenuPaquetFinCondition);
+struct ContenuPaquetFinCondition : public ContenuPaquet
+{
+	explicit ContenuPaquetFinCondition(const size_t position)
+	:ContenuPaquet(position) {}
+	virtual ~ContenuPaquetFinCondition() {}
+	virtual void verification() override {}
+	virtual void construction(ConfigurationPaquet& configuration) override { configuration._condition = true; }
+	virtual void generation(std::ostream& os) override {};
 };
 
 DECL_SHARED_PTR(Paquet);
@@ -868,7 +944,7 @@ struct Paquet
 	// validation unitaire de chaque element du paquet
 	void verification();
 	// consolidation et recuperation des donnees
-	void construction(const bool debug);
+	void construction(const std::string& saveur, const bool debug, std::set<std::string>& drapeaux);
 	// generation du script d'installation
 	void generation(const std::string& fichier_sortie);
 
@@ -877,29 +953,15 @@ struct Paquet
 		return _nom + "-" + _version + "-installer.sh";
 	}
 
-	friend std::ostream& operator<<(std::ostream& os, const Paquet& p)
-	{
-		os	<< "Paquet " << p._nom << std::endl
-			<< "Licence " << p._licence << std::endl
-			<< "Options:" << std::endl;
-		for(const auto& o: p._options)
-		{
-			os << "\t" << (*o) << std::endl;
-		}
-		os	<< "Contenu:" << std::endl;
-		for(const auto& c: p._contenus)
-		{
-			os << "\t" << (*c) << std::endl;
-		}
-		return os;
-	}
 	bool _debug;
 
 	std::string _nom;
 	std::string _licence;
 	std::vector<OptionPaquetSP> _options;
 	std::vector<ContenuPaquetSP> _contenus;
+	std::set<std::string> _drapeaux;
 
+	std::string _saveur;
 	bool _test_version_api_svm = true;
 	std::string _version;
 	std::string _auteur;
